'use strict'

const childProcess = require('child_process')
const path = require('path')

const Boom = require('boom')
const _ = require('lodash')
const Promise = require('bluebird')
const handlebars = require('handlebars')
const fsExtra = require('fs-extra')
const fs = require('fs')

const paths = require('./paths')
const config = require('./config')
const options = require('./options')
const istanbulMocha = require('./istanbul-mocha')
const baseDockerCompose = require('./templates/docker-compose.json')
const dockerState = require('./docker-state')
const commands = require('./commands')
const tracer = require('./tracer')
const processLogs = require('./process-logs')

const DOCKER_PATH = path.join('.narval', 'docker')
const DOCKER_RESOURCES_PATH = 'docker-resources'
const INSTALL_RESOURCES_PATH = 'install-resources'

const baseProcessOptions = {
  cwd: paths.cwd.resolve(DOCKER_PATH),
  encoding: 'utf8',
  stdio: [0, 1, 2],
  shell: true,
  windowsHide: true
}

const copyResources = function (destAbsolutePath) {
  return fsExtra.copy(paths.package.resolve('lib', DOCKER_RESOURCES_PATH), destAbsolutePath)
}

const copyInstallResource = function (pathToCopy, destAbsolutePath) {
  const PATH_SEP = '/'
  const normPath = path.normalize(pathToCopy).replace(/\\/g, PATH_SEP)
  const folderTree = normPath.split('/')
  const fileName = folderTree.pop()
  let relativeDestPath = destAbsolutePath
  if (folderTree.length) {
    relativeDestPath = path.join(destAbsolutePath, folderTree.join(PATH_SEP))
  }
  return paths.cwd.ensureDir(relativeDestPath)
    .then((destPath) => {
      const destFileName = path.join(destPath, fileName)
      return fsExtra.copy(paths.cwd.resolve(normPath), destFileName, {
        dereference: true,
        overwrite: true
      })
    })
}

const copyInstallResources = function (pathsToCopy, destAbsolutePath) {
  pathsToCopy = pathsToCopy || []
  return Promise.mapSeries(pathsToCopy, (pathToCopy) => {
    return copyInstallResource(pathToCopy, destAbsolutePath)
  })
}

const copyInstallScript = function (pathToCopy, destAbsolutePath) {
  if (pathToCopy) {
    return copyInstallResource(pathToCopy, destAbsolutePath)
  }
  return Promise.resolve()
}

const createImageFiles = function (templates, imageData) {
  const imagePath = path.join(DOCKER_PATH, imageData.name)
  const installResourcesPath = paths.cwd.resolve(path.join(imagePath, INSTALL_RESOURCES_PATH))
  const scriptsPath = path.join(imagePath, 'docker-resources', '.narval', 'scripts')

  return Promise.all([
    paths.cwd.ensureDir(scriptsPath),
    paths.cwd.ensureDir(installResourcesPath),
    paths.cwd.ensureDir(imagePath, 'docker-resources', '.narval', 'scripts'),
    paths.cwd.ensureDir('.coverage')
  ])
    .then((absolutePath) => {
      return Promise.all([
        paths.cwd.writeFile(path.join(imagePath, 'Dockerfile'), templates.dockerFile(imageData)),
        paths.cwd.writeFile(path.join(scriptsPath, 'install.sh'), templates.install(imageData)),
        copyResources(paths.cwd.resolve(path.join(imagePath, DOCKER_RESOURCES_PATH))),
        copyInstallResources(imageData.add, installResourcesPath),
        copyInstallScript(imageData.install, installResourcesPath)
      ])
    })
}

const createDockerImagesFiles = function (data) {
  return Promise.map(data.config.dockerImages, (imageData) => {
    return createImageFiles(data.templates, imageData)
  })
}

const serviceNameToVarName = function (name) {
  return name && name.replace(/-/g, '_')
}

const getCustomEnvVars = function (suitesByType) {
  let customVars = []
  _.each(suitesByType, (suiteType) => {
    _.each(suiteType.suites, (suite) => {
      if (suite.test && suite.test.docker && suite.test.docker.env) {
        _.each(suite.test.docker.env, (value, key) => {
          customVars.push(key)
        })
      }
      if (suite.before && suite.before.docker && suite.before.docker.env) {
        _.each(suite.before.docker.env, (value, key) => {
          customVars.push(key)
        })
      }
      _.each(suite.services, (service) => {
        if (service.docker && service.docker.env) {
          _.each(service.docker.env, (value, key) => {
            customVars.push(key)
          })
        }
      })
    })
  })
  return _.uniq(customVars)
}

const getCustomDockerEnvVars = function (suites, serviceVarName) {
  const customEnvVars = getCustomEnvVars(suites)
  let customDockerEnvVars = {}
  _.each(customEnvVars, varKey => {
    customDockerEnvVars[varKey] = '${' + serviceVarName + '_' + varKey + '}'
  })
  return customDockerEnvVars
}

const createComposeFile = function (data) {
  return paths.cwd.ensureDir(DOCKER_PATH)
    .then(() => {
      let compose = _.clone(baseDockerCompose)
      _.each(data.config.dockerContainers, dockerContainer => {
        const serviceVarName = serviceNameToVarName(dockerContainer.name)
        let serviceProps = {
          build: {
            context: './' + dockerContainer.build
          },
          volumes: [{
            type: 'volume',
            source: 'shared',
            target: '/narval/.shared'
          },
          {
            type: 'bind',
            source: '../../.coverage',
            target: '/narval/.coverage'
          },
          {
            type: 'bind',
            source: '../../.narval/logs',
            target: '/narval/.narval/logs'
          }],
          environment: _.extend({
            command_to_run: '${' + serviceVarName + '_command}',
            command_params: '${' + serviceVarName + '_command_params}',
            coverage_options: '${' + 'coverage_options}',
            coverage_enabled: '${' + serviceVarName + '_coverage_enabled}',
            wait_on: '${' + serviceVarName + '_wait_on}',
            exit_after: '${' + serviceVarName + '_exit_after}',
            narval_suite_type: '${' + serviceVarName + '_narval_suite_type}',
            narval_suite: '${' + serviceVarName + '_narval_suite}',
            narval_service: '${' + serviceVarName + '_narval_service}',
            narval_is_docker: '${' + serviceVarName + '_narval_is_docker}'
          }, getCustomDockerEnvVars(data.config.suitesByType, serviceVarName))
        }
        _.each(dockerContainer.bind, (relativePath) => {
          serviceProps.volumes.push({
            type: 'bind',
            source: `../../${relativePath}`,
            target: `/narval/${relativePath}`
          })
        })
        compose.services[dockerContainer.name] = serviceProps
      })
      return paths.cwd.writeFile(path.join(DOCKER_PATH, 'docker-compose.json'), JSON.stringify(compose, null, 2))
    })
}

const getTemplate = function (templateName) {
  return paths.package.readFile('lib', 'templates', templateName)
    .then(fileContent => {
      return Promise.resolve(handlebars.compile(fileContent))
    })
}

const getTemplates = function () {
  return Promise.props({
    dockerFile: getTemplate('Dockerfile.hbs'),
    install: getTemplate('install.hbs')
  })
}

const runCreateFiles = function () {
  return Promise.props({
    config: config.get(),
    templates: getTemplates()
  }).then((data) => {
    return Promise.all([
      createComposeFile(data),
      createDockerImagesFiles(data)
    ])
  })
}

const createFiles = function () {
  let createdFilesPromise = dockerState.createdFiles()
  if (!createdFilesPromise) {
    createdFilesPromise = dockerState.createdFiles(runCreateFiles())
  }
  return createdFilesPromise
}

const waitOnConfigToCommandLine = function (waitOnConfig) {
  let options = []

  options.push(waitOnConfig.timeout ? `--timeout=${waitOnConfig.timeout}` : null)
  options.push(waitOnConfig.delay ? `--delay=${waitOnConfig.delay}` : null)
  options.push(waitOnConfig.interval ? `--interval=${waitOnConfig.interval}` : null)
  options.push(waitOnConfig.reverse ? '--reverse' : null)
  options = options.concat(waitOnConfig.resources)

  return _.compact(options).join(' ')
}

const getWaitOn = function (test) {
  const waitOnConfig = test.docker['wait-on'] || ''
  if (_.isString(waitOnConfig)) {
    return waitOnConfig
  }
  return waitOnConfigToCommandLine(waitOnConfig)
}

const getExitAfter = function (test) {
  return test.docker.exit_after || ''
}

const coverageIsEnabled = function (suite, serviceName, def) {
  if (!suite.coverage) {
    return def || false
  }
  if (suite.coverage.enabled === false) {
    return false
  }
  if (suite.coverage.from) {
    return suite.coverage.from === serviceName
  }
  return def || false
}

const extendProcessEnvVars = function (vars) {
  return _.extend({}, process.env, vars)
}

const addEnvironmentContainerVars = function (object, containerVarName, values) {
  _.each(values, (value, key) => {
    object[`${containerVarName}_${key}`] = value
  })
}

const composeEmptyEnvVars = function () {
  return config.get()
    .then((configuration) => {
      const customEnvVars = getCustomEnvVars(configuration.suitesByType)
      const customEnvVarsObj = {}
      let envVars = {
        coverage_options: ''
      }
      _.each(customEnvVars, (varName) => {
        customEnvVarsObj[varName] = ''
      })

      _.each(configuration.dockerContainers, dockerContainer => {
        const serviceVarName = serviceNameToVarName(dockerContainer.name)
        addEnvironmentContainerVars(envVars, serviceVarName, _.extend({
          command: '',
          command_params: '',
          coverage_enabled: '',
          wait_on: '',
          exit_after: '',
          narval_suite_type: '',
          narval_suite: '',
          narval_service: '',
          narval_is_docker: ''
        }, customEnvVarsObj))
      })
      return Promise.resolve(extendProcessEnvVars(envVars))
    })
}

const composeEnvVars = function (suite, suiteTypeName) {
  return config.get()
    .then((configuration) => {
      let vars = {
        coverage_options: istanbulMocha.istanbul.params(suite, suiteTypeName)
      }
      const testDockerContainer = suite.test.docker && suite.test.docker.container
      const testCoverageIsEnabled = coverageIsEnabled(suite, 'test', true)
      const testCommand = 'narval-default-test-command'
      let testCommandParams = istanbulMocha.mocha.params(suite)

      if (!testDockerContainer) {
        return Promise.reject(Boom.notFound('No docker configuration found for test'))
      }

      let testVarName = serviceNameToVarName(testDockerContainer)

      if (testCoverageIsEnabled) {
        testCommandParams = '-- ' + testCommandParams
      }

      addEnvironmentContainerVars(vars, testVarName, _.extend({
        command: testCommand,
        command_params: testCommandParams,
        coverage_enabled: testCoverageIsEnabled,
        wait_on: getWaitOn(suite.test),
        exit_after: getExitAfter(suite.test)
      }, suite.test.docker.env || {}, {
        narval_suite_type: suiteTypeName,
        narval_suite: suite.name,
        narval_service: 'test',
        narval_is_docker: true
      }))

      _.each(suite.services, (service) => {
        const commandAndParams = istanbulMocha.getCommandAndParams(service.docker.command)
        const varName = serviceNameToVarName(service.docker.container)
        const serviceCoverageIsEnabled = coverageIsEnabled(suite, service.name)
        let exitAfter = getExitAfter(service)

        if (serviceCoverageIsEnabled && commandAndParams.params.length) {
          commandAndParams.params = '-- ' + commandAndParams.params
        }
        addEnvironmentContainerVars(vars, varName, _.extend({
          command: commandAndParams.command,
          command_params: commandAndParams.params,
          coverage_enabled: coverageIsEnabled(suite, service.name),
          wait_on: getWaitOn(service),
          exit_after: exitAfter
        }, service.docker.env || {}, {
          narval_suite_type: suiteTypeName,
          narval_suite: suite.name,
          narval_service: service.name,
          narval_is_docker: true
        }))
      })

      return Promise.resolve(extendProcessEnvVars(vars))
    })
}

const processOptions = function (extraOptions) {
  return (_.extend({}, baseProcessOptions, extraOptions))
}

const runComposeSync = function (command, options) {
  return new Promise((resolve, reject) => {
    tracer.log(`Running Docker command "docker-compose ${command}"`)
    try {
      childProcess.execSync(`docker-compose -f docker-compose.json ${command}`, processOptions(options))
      resolve()
    } catch (error) {
      reject(Boom.badImplementation(`Docker compose command "${command}" failed`))
    }
  })
}

const downVolumes = function () {
  if (!dockerState.executed()) {
    return Promise.resolve()
  }
  return composeEmptyEnvVars()
    .then((vars) => {
      return runComposeSync('down --volumes', {
        env: vars
      })
    })
}

const runServiceContainer = function (serviceDockerConfig, suiteData, envVars) {
  tracer.debug(`Starting docker service "${suiteData.service}" of suite "${suiteData.suite}"`)
  return runComposeSync(`start ${serviceDockerConfig.container}`, {
    env: envVars
  })
    .then(() => {
      return new Promise((resolve, reject) => {
        let logs
        const proc = childProcess.spawn('docker-compose', [
          '-f',
          'docker-compose.json',
          'logs',
          '-f',
          serviceDockerConfig.container
        ], {
          cwd: paths.cwd.resolve(DOCKER_PATH),
          env: envVars
        })

        logs = new processLogs.Handler(proc, {
          type: suiteData.type,
          suite: suiteData.suite,
          service: suiteData.service
        })

        resolve({
          service: suiteData.service,
          container: serviceDockerConfig.container,
          process: proc,
          logs: logs,
          abortOnError: serviceDockerConfig.abortOnError,
          abortOnExit: serviceDockerConfig.abortOnExit
        })
      })
    })
}

const getContainerAndRun = function (suite, suiteTypeName, serviceName, envVars) {
  let serviceConfig
  let coverageFromService = suite.coverage && suite.coverage.from && suite.coverage.from !== 'test'

  if (serviceName === 'test' && suite.test && suite.test.docker && suite.test.docker.container) {
    serviceConfig = _.extend({}, suite.test.docker, {
      abortOnError: true,
      abortOnExit: !coverageFromService
    })
  } else {
    _.each(suite.services, (service) => {
      let isCoveraged = coverageIsEnabled(suite, service.name)
      if (service.name === serviceName && service.docker && service.docker.container) {
        serviceConfig = _.extend({}, service.docker, {
          abortOnError: isCoveraged ? true : service['abort-on-error'],
          abortOnExit: isCoveraged
        })
      }
    })
  }

  if (serviceConfig) {
    return runServiceContainer(serviceConfig, {
      type: suiteTypeName,
      service: serviceName,
      suite: suite.name
    }, envVars)
  }
  if (serviceName === 'test') {
    return Promise.reject(Boom.badImplementation(`There is no Docker configuration for test in suite "${suite.name}" of type "${suiteTypeName}"`))
  }
  tracer.warn(`There is no Docker configuration for service "${serviceName}" in suite "${suite.name}" of type "${suiteTypeName}"`)
  return Promise.resolve()
}

const stopServiceContainer = function (dockerService, envVars) {
  tracer.debug(`Stopping Docker service "${dockerService.service}"`)
  return runComposeSync(`stop ${dockerService.container}`, {
    env: envVars
  })
}

const getExitCode = function (log) {
  let result = /code\s*(\d*)/g.exec(log)
  return (result && result[1]) || '1'
}

const writeCloseLog = function (service, suite, type, code) {
  const filePath = paths.cwd.resolve('.narval', 'logs', type, suite, service)
  const fileFolder = path.join(filePath, 'exit-code.log')
  fsExtra.ensureDirSync(filePath)
  fs.writeFileSync(fileFolder, code)
}

const runServicesAndTest = function (suite, suiteTypeName, envVars) {
  return options.get()
    .then(opts => {
      return runComposeSync('down', {
        env: envVars
      }).then(() => {
        let build = ''
        if (opts.build && !dockerState.built()) {
          dockerState.built(true)
          build = ' --build'
        }
        return runComposeSync(`up --no-start${build}`, {
          env: envVars
        }).then(() => {
          return Promise.map(suite.services || [], (service) => {
            return getContainerAndRun(suite, suiteTypeName, service.name, envVars)
          }).then(startedServices => {
            startedServices = _.compact(startedServices)
            return getContainerAndRun(suite, suiteTypeName, 'test', envVars)
              .then((startedTest) => {
                return new Promise((resolve, reject) => {
                  let stopAllExecuted = false
                  let suiteError

                  startedServices.push(startedTest)

                  const checkStopped = function () {
                    let notStopped = []
                    _.each(startedServices, (serviceData) => {
                      if (!serviceData.closed) {
                        notStopped.push(serviceData.service)
                      }
                    })
                    if (notStopped.length < 1) {
                      if (suiteError) {
                        reject(suiteError)
                      } else {
                        resolve()
                      }
                    } else if (notStopped.length > 1) {
                      tracer.debug(`Services "${notStopped.join(', ')}" are still running. Waiting...`)
                    } else {
                      tracer.debug(`Service "${notStopped[0]}" is still running. Waiting...`)
                    }
                  }

                  const stopAll = function (error) {
                    stopAllExecuted = true
                    suiteError = error
                    tracer.debug('Stopping all docker services')
                    Promise.map(startedServices, (service) => {
                      return stopServiceContainer(service, envVars)
                    })
                      .then(() => {
                        return runComposeSync('down', {
                          env: envVars
                        })
                      })
                      .then(() => {
                        checkStopped()
                      })
                  }

                  _.each(startedServices, (serviceData) => {
                    serviceData.logs.on('close', (logData) => {
                      let exitCode = getExitCode(logData.lastLog)
                      let error
                      let traceMethod = 'debug'

                      serviceData.closed = true
                      writeCloseLog(serviceData.service, suite.name, suiteTypeName, exitCode)
                      if (exitCode !== '0') {
                        traceMethod = 'warn'
                        if (serviceData.abortOnError && !stopAllExecuted) {
                          error = Boom.badImplementation(`Docker service exited with code ${exitCode}`)
                          traceMethod = 'error'
                        }
                      }

                      tracer[traceMethod](`Docker container "${serviceData.container}" of service "${serviceData.service}" exited with code "${exitCode}"`)
                      if ((error || serviceData.abortOnExit) && !stopAllExecuted) {
                        stopAll(error)
                      } else {
                        checkStopped()
                      }
                    })
                  })
                })
              })
          })
        })
      })
    })
}

const runDownVolumes = function (suite) {
  return suite.before && suite.before.docker && suite.before.docker['down-volumes'] ? downVolumes() : Promise.resolve()
}

const runBefore = function (suite, suiteTypeName) {
  if (suite.before && suite.before.docker && suite.before.docker.command) {
    tracer.debug(`Executing before command "${suite.before.docker.command}"`)
    return commands.run(suite.before.docker.command, {
      suiteType: suiteTypeName,
      suite: suite.name,
      service: 'before',
      sync: true,
      env: _.extend({
        narval_suite_type: suiteTypeName,
        narval_suite: suite.name,
        narval_service: 'before',
        narval_is_docker: true
      },
      (suite.before && suite.before.docker && suite.before.docker.env) || {}
      )
    })
  }
  return Promise.resolve()
}

const before = function (suite, suiteTypeName) {
  return runBefore(suite, suiteTypeName)
    .then(() => {
      return runDownVolumes(suite)
    })
}

const run = function (suite, suiteTypeName) {
  dockerState.executed(true)
  return composeEnvVars(suite, suiteTypeName)
    .then((envVars) => {
      return before(suite, suiteTypeName)
        .then(() => {
          return runServicesAndTest(suite, suiteTypeName, envVars)
        })
    })
}

module.exports = {
  createFiles: createFiles,
  downVolumes: downVolumes,
  run: run
}
